import argparse
import os
import sys
import traceback
import datetime
import uuid
# from logger import logger
#from configurator import configurator,validator
from data_compare.data import database,data
from data_compare.core import comparator
from multiprocessing import Pool, Process


class DVM():
    """Usinf this class only for web based request , will merge it with ain later on"""
    def __init__(self, config_data):
        self.config_data = config_data
    def set_config(self):
        pass
    def validate_config(self):
        pass
    def get_src_data(self):
        pass
    def get_tgt_data(self):
        pass
    def get_metadata_db_connection(self):
        pass
    def close_metadata_db_connection(self):
        pass
    def get_results(self):
        try:
            res_dict = {}
            #selecting only execute = yes comparison
            config_list = [(uuid.uuid4(),self.config_data)]
            if config_list:
                process_no = len(config_list)
                pool = Pool(processes=process_no)
                for res in pool.imap_unordered(worker_process,config_list):
                    res = list(res)
                    if res[0] == 0:
                        res_dict["status"]='PASS'
                        r_val,key_only_in_src,key_only_in_tgt,common_keys,diff_count,match_count = [res[i] for i in (0, 1, 2, 3, 4, 5)]
                        res_dict["comp_status"] = "FAIL"
                        if diff_count==0 and len(key_only_in_src)==0 and len(key_only_in_tgt)==0:
                            print("Comparison passed for comp_id {}".format(res[1]))
                            res_dict["comp_status"] = "PASS"
                            res_dict["match_count"] = match_count
                        elif diff_count>0:
                            print("Diff found {}".format(diff_count))
                            res_dict["diff"]=diff_count
                            res_dict["match_count"] = match_count
                        elif len(key_only_in_src)>0:
                            print("Extra key found in src {}".format(len(key_only_in_src)))
                            res_dict['key_only_in_src']=len(key_only_in_src)
                        elif len(key_only_in_tgt)>0:
                            print("Extra key found in tgt {}".format(len(key_only_in_tgt)))
                            res_dict['key_only_in_tgt']=len(key_only_in_tgt)
                    else:
                        res_dict["status"] = "FAIL"
                        print("Comparison failed for comp_id {} ".format(res[1]))
            else:
                print("No executable test case found")
            print("Exiting from get_results")
        except Exception:
            print("Exception in get_results {}".format(traceback.format_exc()))
        return res_dict    



def parseCmdLine():
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("-c","--config",help="Config File Path")
        parser.add_argument("-db", "--dbfile", help="Database File")
        parser.add_argument("-sdb","--srcDB",help="Source Database Name")
        parser.add_argument("-st","--srcTab",help="Source Table")
        parser.add_argument("-ss","--srcSchema",help="Source Schema")
        parser.add_argument("-tdb","--tgtDB",help="Target Database")
        parser.add_argument("-tt","--tgtTab",help="target Table")
        parser.add_argument("-ts","--tgtSchema",help="target schema")
        parser.add_argument("-sq","--srcQuery",help="Source Query")
        parser.add_argument("-tq","--tgtQuery",help="Target Query")
        parser.add_argument("-sqf","--srcQueryFile",help="Path of source query file")
        parser.add_argument("-tqf","--tgtQueryFile",help="Path of target query file")
        parser.add_argument("-pk","--primaryKeys",help="primary key list separated with double hyphen(--)")
        args = parser.parse_args()
    except Exception:
        args = None
        print("Exception occured in parseCmdLine {}".format(traceback.format_exc()))
    return args


        
def checkCmdLineParams(args): 
    try:
        r_val = 1
        if args.config and args.dbfile:
            r_val = 0
        else:
            print("Config file or DB file not passed checking commandline variable")
            paramMissing =False
            if not args.srcDB:
                print("SrcDB not passed. Exiting")
                paramMissing = True
            if not args.tgtDB:
                print("tgtDB not passed. Exiting")
                paramMissing = True
            if not args.srcQueryFile:
                print("srcQueryFile is not passed.checking for srcQuery")
                if not args.srcQuery:
                    paramMissing = True
                    print("srcQuery is also not found .exiting")
            if not args.tgtQueryFile:
                print("tgtQueryFIle is not passed . checking for tgtQuery")
                if not args.tgtQuery:
                    paramMissing = True
                    print('tgtQuery is also passed.exiting')
            if not args.primaryKeys:
                paramMissing = True
                print("primaryKeys not passed.exiting")
            if not paramMissing:
                r_val = 0
        if r_val ==0 :
            print("Check cmd line passed")
    except Exception:
        print("Exception in checkCmdLineParam {}".format(traceback.format_exc()))
    return r_val


def worker_process(config):
    try:
        print(f"config is {config}")
        r_val = 1
        comp_id = config[0]
        print(config[0])
        print("Worker process for cmp_id {} started".format(comp_id))
        comp_info = config[1]
        print(comp_info)
        DataObj = data.Data(comp_info)
        src_data,src_header,tgt_data,tgt_header = DataObj.get_data()
        if src_data and tgt_data:
            #creating comparator object
            comp_obj = comparator.Comparator(comp_info, src_data, tgt_data, src_header, tgt_header)
            res = comp_obj.compare()
        else:
            print("No data found either in src or in tgt. Not Comparing")
            res = (1, None, None, None, None, None)
        print("Worker process running for comp_id {} completed".format(comp_id))
    except Exception:
        print("Exception in worker proces {}".format(traceback.format_exc()))
        res = (1, None, None, None, None, None)
    return res

def main():
    try:
        print("inside main")
        r_val = 1
        args = parseCmdLine()
        r_val = checkCmdLineParams(args)
        if r_val == 0:
            configuratorObj = configurator.Configurator(args)
            r_val = configuratorObj.set_config()
            if r_val == 0:
                config_data, db_data = configuratorObj.get_config()
                print("Config data in main is {}".format(config_data))
                print("DB data in main is {}".format(db_data))
                res_list = []
                #selecting only execute = yes comparison
                config_list = [[x,config_data['comparisons'][x]] for x in config_data["comparisons"] if config_data['comparisons'][x]['execute'].lower()=='yes']	
                print("config_list af is {}".format(config_list))
                for i in config_list:
                    db_src = i[1]['srcDatabase']
                    db_tgt = i[1]['tgtDatabase']
                    db_one = {}
                    db_two = {}
                    for j in db_data[db_src]:
                    #print(i)
                        db_one['src_'+j]=db_data[db_src][j]
                    for j in db_data[db_tgt]:
                        db_two['tgt_'+j]=db_data[db_tgt][j]
                #print("db_src is {}".format(db_one))
                #print("db_tgt is {}".format(db_two)) 
                    i[1] = {**i[1], **db_one, **db_two}
                print("config_list is {}".format(config_list))
                if config_list:
                    process_no = max(len(config_list),2) #taking it 2 for local machine will increase later
                    
                    if process_no>1:
                        pool = Pool(processes = process_no)
                        for res in pool.imap_unordered(worker_process,config_list):
                            res = list(res)
                            print(res)
                            #print(res)
                            if res[0]==0:
                                r_val,key_only_in_src, key_only_in_tgt, common_keys, diff_count, match_count = [res[i] for i in (0, 1, 2, 3, 4, 5)]
                                if diff_count==0 and len(key_only_in_src)==0 and len(key_only_in_tgt)==0:
                                    print("Comparison passed for comp_id {}".format(res[1]))
                                elif diff_count>0:
                                    print("Diff found {}".format(diff_count))
                                elif len(key_only_in_src)>0:
                                    print("Extra key found in src {}".format(key_only_in_src))
                                elif len(key_only_in_tgt)>0:
                                    print("Extra key found in tgt {}".format(key_only_in_tgt))
                            else:
                                print("Comparison failed for comp_id {}".format(res[1]))
                    elif 2==3:
                        #this code is not used
                        #launching a single process for single comparisons
                        proc = Process(target = worker_process, args = config_list)
                        proc.start()
                        proc.join()
                    else:
                        res = worker_process(config_list[0])
                        if res[0]==0:
                                r_val,key_only_in_src, key_only_in_tgt, common_keys, diff_count, match_count = [res[i] for i in (0, 1, 2, 3, 4, 5)]
                                if diff_count==0 and len(key_only_in_src)==0 and len(key_only_in_tgt)==0:
                                    print("Comparison passed for comp_id {}".format(res[1]))
                                elif diff_count>0:
                                    print("Diff found {}".format(diff_count))
                                elif len(key_only_in_src)>0:
                                    print("Extra key found in src {}".format(key_only_in_src))
                                elif len(key_only_in_tgt)>0:
                                    print("Extra key found in tgt {}".format(key_only_in_tgt))
                        else:
                            print("Comparison failed for comp_id {}".format(res[1]))
                else:
                    print("No executable testcase found")
                r_val = 0
    except Exception:
        print("Exception in main {}".format(traceback.format_exc()))

if __name__ == '__main__':
    print("hello world")
    main()



import os
import logging
from typing import Dict
from datetime import datetime, timezone
from itertools import chain
import json
from gempyp.libs.enums.status import status
from gempyp.libs.common import findDuration, dateTimeEncoder
import traceback


class TemplateData:
    def __init__(self, header="Gemini Report"):
        # initliza the data to be stored as a JSON
        self.REPORTDATA = {"Header": header, "steps": []}

    def newReport(self, project_name: str, tescase_name: str):
        metadata = []
        self.kwargs_list=[]
        # 1st Column
        column1 = {
            "TESTCASE NAME": tescase_name,
            "SERVICE PROJECT": project_name,
            "DATE OF EXECUTION": {"value": datetime.now(timezone.utc), "type": "date"},
        }
        metadata.append(column1)

        self.REPORTDATA["meta_data"] = metadata

    def newRow(self, title: str, description: str, status: status, **kwargs):
        step = {"Step Name": title, "Step Description": description, "status": status}
        if not kwargs.get("attachment"):
            kwargs.pop("attachment")
        if kwargs:
            if kwargs not in self.kwargs_list:
                self.kwargs_list.append(kwargs)
        step.update(kwargs)
        self.REPORTDATA["steps"].append(step)

    # finalize the result. Calculates duration etc.
    def finalizeResult(
        self, begin_time: datetime, end_time: datetime, status_counts: Dict
    ):
        # column2
        column2 = {
            "EXECUTION STARTED ON": {"value": begin_time, "type": "datetime"},
            "EXECUTION ENDED ON": {"value": end_time, "type": "datetime"},
            "EXECUTION DURATION": findDuration(begin_time, end_time),
        }

        # column3
        column3 = {k.name: v for k, v in status_counts.items()}
        self.REPORTDATA["meta_data"].append(column2)
        self.REPORTDATA["meta_data"].append(column3)
        # filters
        # self.REPORTDATA["FilterNames"] = self._getFilters()
        # filter_values = {}
        # filter_values["status"] = [value.name for value in status_counts.keys()]
        # self.REPORTDATA["FilterValues"] = filter_values

    """def _getFilters(self) -> Dict:
        
        # return the unique columns

        filter_names = list(
            set(chain.from_iterable(step.keys() for step in self.REPORTDATA["steps"]))
        )
        # filter_names.pop("status")
        filter_dict = {name: "Input" for name in filter_names}
        filter_dict["status"] = "Dropdown"

        return filter_dict"""

    # Converts the data to the JSON
    def _toJSON(self) -> str:
        """
        dump the data in REPORTDATA
        """
        kwargs_list_comm =  dict(j for i in self.kwargs_list for j in i.items())
        lis = kwargs_list_comm.keys()
        for i in self.REPORTDATA["steps"]:
            for j in lis:
                if j in i:
                    continue
                else:
                    i.update({j:"-"})

        try:
            result_data = json.dumps(self.REPORTDATA, cls=dateTimeEncoder)
            return result_data
        except TypeError as error:
            logging.error("Error occured while serializing the testcase Result Data")
            logging.error(f"Error: {error}")
        except Exception as e:
            logging.error("some Error occured")
            logging.error(f"Error: {e}")
        return "Error"
    
    def makeSuiteReport(self, json_data, testcase_data, ouput_folder):
        """
        saves the report json 
        """
        suiteReport = None
        date = datetime.now().strftime("%Y_%b_%d_%H%M%S_%f")

        suite_path = os.path.dirname(__file__)
        suite_path = os.path.join(os.path.split(suite_path)[0], "final_report.html")
        with open(suite_path, "r") as f:
            suiteReport = f.read()

        reportJson = json_data
        reportJson = json.loads(reportJson)
        reportJson["teststep_details"] = testcase_data
        repJson = reportJson
        # self.testcaseData = json.dumps(self.testcaseData)
        reportJson = json.dumps(reportJson)
        suiteReport = suiteReport.replace("DATA_1", reportJson)

        ResultFile = os.path.join(ouput_folder, "Result_{}.html".format(date))
        ouput_file_path = ResultFile
        with open(ResultFile, "w+") as f:
            f.write(suiteReport)
        return repJson, ouput_file_path
    
    def makeTestcaseReport(self):
        index_path = os.path.dirname(__file__)
        index_path = os.path.join(os.path.split(index_path)[0], "testcase.html")
        with open(index_path, "r") as f:
            Result_data = f.read()
        json_data = self._toJSON()
        return json.loads(json_data)

    def repSummary(self, repJson, output_file_path, jewel_link, failed_testcases, unuploaded_path):
        """
        logging some information
        """
        try:
            logging.info("---------- Finalised the report --------------")
            logging.info("============== Run Summary =============")
            count_info = {key.lower(): val for key, val in repJson['suits_details']['testcase_info'].items()}
            log_str = f"Total Testcases: {str(count_info.get('total', 0))} | Passed Testcases: {str(count_info.get('pass', 0))} | Failed Testcases: {str(count_info.get('fail', 0))} | "
            status_dict = {"info": "Info", "warn": "WARN", "exe": "Exe", "err":"ERR"}
            for key, val in count_info.items():
                if key in status_dict.keys():
                    log_str += f"{status_dict[key.lower()]} Testcases: {val} | "
            logging.info(log_str.strip(" | "))

            if failed_testcases != 0:
                logging.info(f"Number of Testcases not Uploaded:{failed_testcases}")
                logging.info(f"Unuploaded Testcase File Path:{unuploaded_path}")
            
            if len(jewel_link)>0:
                logging.info('Report at Jewel: {link}'.format(link = jewel_link))
            logging.info('-------- Report created Successfully at: {path}'.format(path=output_file_path))


        except Exception as e:
            logging.error(traceback.format_exc(e))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gempyp',
 'gempyp.config',
 'gempyp.data_compare',
 'gempyp.data_compare.common',
 'gempyp.data_compare.configurator',
 'gempyp.data_compare.core',
 'gempyp.data_compare.data',
 'gempyp.data_compare.report',
 'gempyp.data_compare.tools',
 'gempyp.dv',
 'gempyp.engine',
 'gempyp.engine.executors',
 'gempyp.jira',
 'gempyp.libs',
 'gempyp.libs.enums',
 'gempyp.libs.exceptions',
 'gempyp.pyprest',
 'gempyp.reporter',
 'gempyp.sdk']

package_data = \
{'': ['*'], 'gempyp.data_compare': ['config/*']}

install_requires = \
['XlsxWriter>=3.0.3,<4.0.0',
 'certifi>=2022.6.15,<2023.0.0',
 'cffi>=1.15.0,<2.0.0',
 'charset-normalizer>=2.0.12,<3.0.0',
 'coloredlogs>=15.0.1,<16.0.0',
 'cryptography>=38.0.1,<39.0.0',
 'humanfriendly>=10.0,<11.0',
 'idna>=3.4,<4.0',
 'lxml>=4.9.0,<5.0.0',
 'mysql-connector-python>=8.0.30,<9.0.0',
 'ntlm-auth>=1.5.0,<2.0.0',
 'numpy>=1.19.5,<2.0.0',
 'pandas>=1.1.5,<2.0.0',
 'pg8000>=1.26.0,<2.0.0',
 'pycparser>=2.21,<3.0',
 'pyreadline3>=3.4.1,<4.0.0',
 'pyreadline>=2.1,<3.0',
 'pytest>=6.2.4,<7.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'pytz>=2022.2.1,<2023.0.0',
 'requests-ntlm>=1.1.0,<2.0.0',
 'requests>=2.27.1,<3.0.0',
 'six>=1.16.0,<2.0.0',
 'urllib3>=1.26.12,<2.0.0']

entry_points = \
{'console_scripts': ['gempyp = gempyp.gemPyp:main']}

setup_kwargs = {
    'name': 'gempyp',
    'version': '1.0.3',
    'description': 'An ecosystem of libraries useful for software development',
    'long_description': '<h1 align="center">\n\t<img\n\t\twidth="350"\n\t\talt="GemPyp"\n\t\tsrc="https://gempyp.gemecosystem.com/static/media/Gempyp.e73184c67dca4df150fa37048866ecb6.svg">\n</h1>\n\n<h3 align="center">\n\t<div><h2>GemPyP</h2></div>\n    An out of the box Python Automation Framework\n</h3>\n\n<p align="center">\n\t<strong>\n\t\t<a href="https://gempyp.gemecosystem.com/">Website</a>\n\t\t•\n\t\t <a href="##">Docs</a>\n\t\t•\n\t\t <a href="https://github.com/Gemini-Solutions/gempyp/">Repo</a>\n\t</strong>\n</p>\n\n<p align="center">\n<img src="https://img.shields.io/badge/python-3.6-blue"/>\n<img src="https://img.shields.io/pypi/dw/gempyp">\n<img src="https://img.shields.io/pypi/v/gempyp?color=red&label=version&logo=gempyp">\n<img src="https://img.shields.io/pypi/implementation/gempyp">\n\n</p>\n\n## Table of contents[![]()](#table-of-contents)\n\n1. [Overview](#overview)\n2. [Installation](#installation)\n3. [Requirements](#requirements)\n4. [Features](#features)\n5. [Usage](#usage)\n6. [Documentations](#docs)\n7. [Contributors](#contributors)\n8. [More](#more)\n9. [Credits](#credits)\n\n## Overview[![]()](#overview)\n\nGemPyP is a testing and reporting framework that allows automatic execution of testcases along with generation of the report, that enables effortless analysis and monitoring of the set.\n\nThe setting and configurations, including the testcases are passed as a config file to the framework, then GemPyP handles the environments, execution, reporting, storing data and miscellaneous.\n\nIt allows-\n\n- Email integration\n- Parallel execution of testcases\n- Customization of test Suite reports\n- Platform independence (Supports Linux, Windows and MacOS)\n\n## Installation[![]()](#installation)\n\nFor installation make sure require version of python and pip is installed in the system.\n\n```powershell\n$ pip install gempyp\n```\n\n## Requirements[![]()](#requirements)\n\n### Python version 3.6 or above\n\n### Required Libraries\n\n- certifi==2022.6.15\n- cffi==1.15.0\n- charset-normalizer==2.0.12\n- coloredlogs==15.0.1\n- cryptography==37.0.2\n- humanfriendly==10.0\n- idna==3.3\n- lxml==4.9.0\n- ntlm-auth==1.5.0\n- numpy==1.22.4\n- pandas==1.4.2\n- pycparser==2.21\n- pyreadline3==3.4.1\n- python-dateutil==2.8.2\n- pytz==2022.1\n- requests==2.28.0\n- requests-ntlm==1.1.0\n- six==1.16.0\n- urllib3==1.26.9\n\n### These libraries will get downloaded automatically with the package.\n\n## Features[![]()](#features)\n\n1. Easy to use.\n2. Cross Platform.\n3. Multiple modes of execution.\n4. Static Report generated after execution.\n5. Saves Time\n\n## Usage[![]()](#usage)\n\n### Can be used in API Automation and testing python testcases.\n\n## Documentations[![]()](#docs)\n\nThe official documentation is hosted on Read the Docs: https://gempyp.readthedocs.io/en/latest/\n\n## Contributors[![]()](#contributors)\n\nWe welcome and recognize all contributions. You can see a list of current contributors in the [contributors tab](https://github.com/Gemini-Solutions/gempyp/).\n\n## More[![]()](#more)\n\nFor more information visit https://gempyp.gemecosystem.com/ and https://gemecosystem.com/\n\n### Credits[![]()](#credits)\n\n- TODO\n',
    'author': 'Gemini Solutions-QA',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Gemini-Solutions/gempyp',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.8,<4.0.0',
}


setup(**setup_kwargs)



import pandas as pd
import traceback
import json
from gempyp.libs.common import dateTimeEncoder, findDuration
from datetime import datetime, timezone
import numpy as np


class TestData:
    def __init__(self):
        """
        declairing some attribute the are used in testcaseDetails in report and return a object that is used to update data
        """
        self.testcaseDetailColumn = [
            "tc_run_id",
            "start_time",
            "end_time",
            "name",
            "category",
            "log_file",
            "status",
            "steps",
            "machine",
            "result_file",
            "product_type",
            "ignore",
            "meta_data",
            "user_defined_data",
            "base_user",
            "invoke_user",
            "run_type",
            "run_mode"
        ]
        self.misc_detail_column = ["run_id", "key", "value", "table_type"]

        # can have anyamount of columns
        # this should always have one row so it can be made a dict or something instad of a dataframe
        self.suite_detail = pd.DataFrame()

        self.testcase_details = pd.DataFrame(columns=self.testcaseDetailColumn)
        self.misc_details = pd.DataFrame(columns=self.misc_detail_column)

    def toSuiteJson(self):
        """
        converts the dataframe to Json
        used in uploadsuitedata (run() method in engine.py)
        """
        if self.suite_detail.empty:
            return {}
        
        self.suite_detail = self.suite_detail.replace(np.nan, '-', regex=True)
        data = self.suite_detail.to_dict(orient="records")[0]
        misc_data = self.misc_details[
            self.misc_details["table_type"].str.upper() == "SUITE"
        ]
        # the above misc data is not being used

        data["meta_data"] = data["meta_data"]
        return json.dumps(data, cls=dateTimeEncoder)

    def totestcaseJson(self, tc_run_id, s_run_id):
        """
        returns the json for testcasedata
        used in update_df method of engine.py
        """

        self.testcase_details = self.testcase_details.replace(np.nan, '-', regex=True)
        test_data = self.testcase_details.loc[
            self.testcase_details["tc_run_id"].str.upper() == tc_run_id
        ]
        if test_data.empty:
            return {}
        test_data = test_data.to_dict(orient="records")[0]
        misc_data = self.misc_details.loc[self.misc_details["run_id"].str.upper() == tc_run_id]
        misc_data = misc_data.to_dict(orient="records")
        test_status = {}
        for step in test_data["steps"]:
            key = step.get("status")
            if test_status.get(key, None) is not None:
                test_status[key] = test_status.get(key) + 1
            else:
                test_status[key] = 1
        test_status["TOTAL"] = sum(test_status.values())
        prio_list = ['TOTAL', 'PASS', 'FAIL']
        sorted_dict = {}
        for key in prio_list:
            if key in test_status:
                sorted_dict[key] = test_status[key]
                test_status.pop(key)
            elif key == "PASS" or key == 'FAIL':
                sorted_dict[key] = 0
        sorted_dict.update(test_status)
        # test_data["duration"] = findDuration(test_data["start_time"], test_data["end_time"])

        test_data["user_defined_data"] = dict()
        # test_data["duration"]="{0:.{1}f} sec(s)".format((test_data["end_time"]-test_data["start_time"]).total_seconds(),2)
        """ Adding misc data to user_defined_data column for each testcase
        Here misc data is only for one testcase.
        {"key1": "value1", "key2": "value2"...}"""
        if len(misc_data) > 0:
            for miscs in misc_data:
                print("--- misc key", miscs.get("key", None))
                key = str(miscs["key"])
                val = str(miscs["value"])
                test_data["user_defined_data"][key] = val

        meta_data = [
            {
                "TESTCASE NAME": test_data["name"], 
                "SERVICE PROJECT": "None", 
                "DATE OF EXECUTION": {"value": datetime.now(timezone.utc), "type": "date"},
                # "RESPONSE TIME":"{0:.{1}f} sec(s)".format((test_data["end_time"]-test_data["start_time"]).total_seconds(),2)
            }, 
            {
                "EXECUTION STARTED ON": {"value": test_data["start_time"], "type": "datetime"},
                "EXECUTION ENDED ON": {"value": test_data["end_time"], "type": "datetime"}, 
                "EXECUTION DURATION": findDuration(test_data["start_time"], test_data["end_time"])
            }, 
            sorted_dict
            # {"response_time":"{0:.{1}f} sec(s)".format((test_data["end_time"]-test_data["start_time"]).total_seconds(),2)}
            ]

        test_data["meta_data"] = meta_data
        test_data["s_run_id"] = s_run_id
        test_data["user_defined_data"]["duration"]="{0:.{1}f} sec(s)".format((test_data["end_time"]-test_data["start_time"]).total_seconds(),2)
        
        return json.dumps(test_data, cls=dateTimeEncoder)

    def _validate(self):
        """
        used by fromJSON to validate the json data
        """
        pass

    def getJSONData(self):
        """
        provide the report json to makereport() method of engine file
        """
        suite_report = {}
        self.suite_detail = self.suite_detail.replace(np.nan, "-", regex=True)
        suite_dict = self.suite_detail.to_dict(orient="records")[0]
        testcase_dict = self.testcase_details.to_dict(orient="records")
        misc_dict = self.misc_details.to_dict(orient="records")
        try:

            # converting testcase_dict to dict for easy parsing
            test_dict = {d['tc_run_id']: d for d in testcase_dict}
            key = list(test_dict.keys())
            key = key[0]
            test_data = test_dict[key]
            test_dict[key] = test_data
            for each_misc in misc_dict:
                test_dict[each_misc['run_id']][each_misc["key"]] = each_misc["value"]

            # converting dict back to list
            testcase_dict = [test_dict[tc_run_id] for tc_run_id in test_dict.keys()]
        except Exception as e:
            traceback.print_exc()
        for i in range(len(testcase_dict)):
            for key in ["meta_data", "base_user", "invoke_user", "user_defined_data"]:
                if(key in testcase_dict[i].keys()):
                    testcase_dict[i].pop(key)
        suite_dict["testcase_details"] = testcase_dict
        testcase_counts = self.getTestcaseCounts()

        prio_list = ['total', 'PASS', 'FAIL']  # to be optimized
        sorted_dict = {}
        for key in prio_list:
            if key in testcase_counts :
                sorted_dict[key] = testcase_counts[key]
                testcase_counts.pop(key)
            elif key == "PASS" or key == 'FAIL':
                sorted_dict[key] = 0
        sorted_dict.update(testcase_counts)
        testcase_counts = sorted_dict
        suite_dict["testcase_info"] = testcase_counts
        suite_report["suits_details"] = suite_dict
        suite_report["report_product"] = "GEMPYP"       
        return json.dumps(suite_report, cls=dateTimeEncoder)

    def getTestcaseCounts(self):
        """
        return the no. of testcases
        """
        group = self.testcase_details.groupby(["status"]).size()
        group = group.to_dict()
        total = 0
        for i in group:
            total += group[i]
        group["total"] = total

        return group



import logging as logger
import gempyp.pyprest.compareFunctions as cf
from gempyp.pyprest.keyCheck import KeyCheck
from gempyp.pyprest import utils
from gempyp.pyprest.utils import getKeyList
from copy import deepcopy
from gempyp.pyprest.legacyComparison import legacyApiComparison
from gempyp.libs.enums.status import status
from gempyp.pyprest.variableReplacement import VariableReplacement as var_replacement


class PostAssertion:
    def __init__(self, pyprest_obj):
        self.pyprest_obj = pyprest_obj
        self.logger = self.pyprest_obj.logger
        self.isLegacyPresent = False
        self.legacy_all_keys = []
        self.all_keys = []
        self.notCompareAll = True
        self.space_handler = {
            "not to": "notto",
            "not in": "notin",
        }
        pass

    def postAssertion(self):
        """This function-
        -parses post assertion string and runs keycheck on the keys for post assertion
        -for the keys that are found in the response, fetches their values
        -gets the comparison result and update logs and reporter obj.
        """
        if self.pyprest_obj.post_assertion:
            self.logger.info("************** INSIDE POST ASSERTION  **************") 
            var_replacement(self.pyprest_obj).variableReplacement()

            #
            if self.pyprest_obj.legacy_res is not None:
                self.logger.info("Legacy API found, proceeding for post assertion accordingly....")
                self.isLegacyPresent = True
                self.legacy_all_keys = getKeyList().getKeys(utils.formatRespBody(self.pyprest_obj.legacy_res.response_body))
                self.all_keys = getKeyList().getKeys(utils.formatRespBody(self.pyprest_obj.res_obj.response_body))
            self.post_assertion_str = " ".join(self.pyprest_obj.post_assertion.split())
            
            if 'COMPARE ALL' in self.post_assertion_str.upper():
                self.notCompareAll = False

            post_ass_str_list = self.post_assertion_str.strip(";").split(";")
            
            key_str = []
            legacy_key_str = []
            # comparison_val_str = []

            assertion_list = self.getAssertionDict(post_ass_str_list)
            for each in assertion_list:
                if 'legacy' in list(each.keys())[0]:
                    legacy_key_str.append(list(each.keys())[0])
                elif list(each.keys())[0].upper() == 'ALL':
                    legacyApiComparison(self.pyprest_obj).compare_all()
                else:
                    key_str.append(list(each.keys())[0])
            key_str = list(set(key_str))
            if len(legacy_key_str)>0:
                 legacy_key_str = list(set(legacy_key_str))
            
            
            if self.isLegacyPresent and len(self.all_keys)==0 and self.notCompareAll:
                self.pyprest_obj.reporter.addRow("Executing Post Assertion check", "Keys for assertion check", status.INFO, CURRENT_API="Keys to execute assertion check in current API are: </br>" + "</br>".join(key_str),LEGACY_API="Keys to execute assertion check in legacy API are: </br>" + "</br>".join(legacy_key_str))
            elif self.isLegacyPresent and len(self.all_keys) > 0 and self.notCompareAll:
                self.pyprest_obj.reporter.addRow("Executing Post Assertion check", "Running Assertion on legacy and current API Responses", status.INFO, CURRENT_API="Assertions for Current API",LEGACY_API="Assertions for Legacy API")
            elif self.notCompareAll:
                self.pyprest_obj.reporter.addRow("Executing Post Assertion check", "Keys to execute assetion check are: </br>" + "</br>".join(key_str), status.INFO)

            self.logger.info("Post assertion string- " + self.post_assertion_str)
            
            
            
            list_of_all_keys = [*list(set(key_str)) , *list(set(legacy_key_str))]
            key_val_dict = {}        
            for each_assert in list_of_all_keys:
                key_part_list = each_assert.split(".") 
                if "legacy" in each_assert:
                    response_json = utils.formatRespBody(self.pyprest_obj.legacy_res.response_body)
                else:
                    response_json = utils.formatRespBody(self.pyprest_obj.res_obj.response_body)
                result = KeyCheck(self.pyprest_obj).findKeys(response_json, deepcopy(key_part_list), deepcopy(key_part_list))
                if result.upper() != "FOUND":
                    self.logger.info("====== Key Not Found in response =======")
                    self.logger.info("'" + each_assert + "' is not found")
                    if self.isLegacyPresent:
                        self.pyprest_obj.reporter.addRow("Executing post assertion on current API ", f"Checking presence of key {each_assert} in response", status.FAIL, CURRENT_API=f"Key {each_assert} is not found in the response",LEGACY_API="-")
                        self.pyprest_obj.reporter.addMisc("REASON OF FAILURE", "Some keys are missing in Response")
                    else:    
                        self.pyprest_obj.reporter.addRow(f"Checking presence of key {each_assert} in response", f"Key {each_assert} is not found in the response", status.FAIL)
                        self.pyprest_obj.reporter.addMisc("REASON OF FAILURE", "Some keys are missing in Response")
                else:
                    key_val_dict = utils.fetchValueOfKey(response_json, key_part_list, result, key_val_dict)
            self.postAssertionFunc(key_val_dict, assertion_list)

    def getAssertionDict(self, string_list):     
        """assertion list-
        [
            {keys: {operator: operator, value: expected value}},
        ]
        """
        # need to add condition for compare all and compare all except in legacy
        temp_str = str(";".join(string_list))
        for i in self.space_handler.keys():
            temp_str = temp_str.replace(str(i), self.space_handler[i])
        string_list = temp_str.split(';')
        flag = 0
        list_ = []
        for each in string_list:
            if "COMPARE" == each.strip(" ").split(" ")[0].upper():
                each = each.strip(" ").split(" ", 1)[1]
                flag = 1
            if flag == 1:
                assert_str = each.strip(" ").split(" ", 1)
                key = assert_str[0]
                opr_val = assert_str[1].strip(" ").split(" ", 1)
                opr = opr_val[0].strip(" ")
                val = opr_val[1].strip(" ")
                list_.append(dict({key: {"operator": "".join(opr.strip(" ")), "value": val.strip(" ")}}))
        return list_

    def postAssertionFunc(self, key_val_dict, assertion_list):
        """This function-
        -removes the element from the assertion dict that are not valid
        -calls comparison functions based on the operator type
        """
        self.logger.info("-----------inside assertion func-----------")

        # removing the keys that are not present
        for ele in assertion_list:
            if list(ele.keys())[0] not in key_val_dict.keys():
                assertion_list.remove(ele)
        key_val_dict_legacy = {}
        # calling compare function for each validation string
        for each_assert in assertion_list:
            result_legacy = ''
            key = list(each_assert.keys())[0]
            operator = list(each_assert.values())[0]['operator']
            value = list(each_assert.values())[0]['value']            
            if self.isLegacyPresent and value.split(".")[-1] in self.legacy_all_keys or value.split(".")[-1] in self.all_keys and '"' not in value and "'" not in value: 
                key_part_list = value.split(".") 
                if 'legacy' in value:
                    result_legacy = KeyCheck(self.pyprest_obj).findKeys(utils.formatRespBody(self.pyprest_obj.legacy_res.response_body), deepcopy(key_part_list), deepcopy(key_part_list))
                    if result_legacy == 'FOUND':
                        key_val_dict_legacy = utils.fetchValueOfKey(utils.formatRespBody(self.pyprest_obj.legacy_res.response_body), key_part_list, result_legacy, key_val_dict_legacy)
                else:
                    result_legacy = KeyCheck(self.pyprest_obj).findKeys(utils.formatRespBody(self.pyprest_obj.res_obj.response_body), deepcopy(key_part_list), deepcopy(key_part_list))
                    if result_legacy == 'FOUND':
                        key_val_dict_legacy = utils.fetchValueOfKey(utils.formatRespBody(self.pyprest_obj.res_obj.response_body), key_part_list, result_legacy, key_val_dict_legacy)              
                                
            elif self.isLegacyPresent and not value.isdigit() and  '"' not in value and "'" not in value:
                if 'legacy' in value :
                    self.pyprest_obj.reporter.addRow(f"Running Assertion on comparing the respective values of {key} and {value}",f"Key not found",status.FAIL, CURRENT_API=f"-", LEGACY_API=f"{value} key missing in legacy response")  
                else:
                    self.pyprest_obj.reporter.addRow(f"Running Assertion on comparing the respective values of {key} and {value}",f"Key not found",status.FAIL, CURRENT_API=f"{value} key missing in Current response", LEGACY_API=f"-")
                    self.pyprest_obj.reporter.addMisc("REASON OF FAILURE", "key for comparison not found in response")

            tolerance = 0.1
            if result_legacy == "FOUND":
                if operator == "notto":
                    cf.compare_notto_resp(self.pyprest_obj.reporter,key, value, key_val_dict, key_val_dict_legacy, result_legacy,tolerance)
                elif operator == "to" :
                    cf.compareToResp(self.pyprest_obj.reporter,key,value,key_val_dict,key_val_dict_legacy, result_legacy, tolerance)
            elif (value.split(".")[-1] not in self.all_keys) and ('legacy' in key and self.isLegacyPresent and len(key_val_dict_legacy)==0):
                self.pyprest_obj.reporter = utils.compare(self.pyprest_obj.reporter, key, operator, value, key_val_dict, tolerance,self.isLegacyPresent, True)
            elif (value.split(".")[-1]) not in self.legacy_all_keys and 'legacy' not in key and self.isLegacyPresent and len(key_val_dict_legacy)>0:
                self.pyprest_obj.reporter = utils.compare(self.pyprest_obj.reporter, key, operator, value, key_val_dict, tolerance, self.isLegacyPresent)
            elif not self.isLegacyPresent:
                self.pyprest_obj.reporter = utils.compare(self.pyprest_obj.reporter, key, operator, value, key_val_dict, tolerance)
                            

                




import json
import logging
from time import time
import traceback
from datetime import datetime
from getpass import getuser
import warnings
from cryptography.fernet import Fernet
import requests
import requests.auth
from requests_ntlm import HttpNtlmAuth
import os


class Api:
    def __init__(self):
        pass

    def execute(self, request):
        encrypt_key = "Y4irRsiBmyGMBie5gAZ8va3IOHVOYZFxC5L1-jNydZk="
        result = Response()
        # if not request.file:
        header_dict = {key.upper(): value.upper() for key, value in request.headers.items()}
        if header_dict.get("CONTENT-TYPE", "") == "APPLICATION/JSON":
                try:
                    if not isinstance(request.body, str):
                        request.body = json.dumps(request.body)
                except Exception as e:
                    print("JSON object can not be serialized")
                    print(str(e))
            # write code for authentication 
            # decrypt password
        auth = None
        try:
                if request.auth.upper() == "PASSWORD":
                    password = self.decrypt_(request.credentials.get("password", ""), encrypt_key)
                    auth = HttpNtlmAuth(request.credentials.get("username", " "), password)
        except Exception as e:
                logging.info("Error occured while creating the auth object- " + str(e))
                auth = None
        try:
                start_time = end_time = datetime.now()
                obj = Response()
                if (
                    request.api == ""
                    or request.method == ""
                    or request.api is None
                    or request.method is None
                ):
                    raise Exception("Api and method are can not be empty or NULL")
                elif str.upper(request.method) == "POST":
                    start_time = datetime.now()
                    if request.auth.upper() == "PASSWORD":
                        resp = requests.post(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            verify=request.SSLVerify,
                            auth=auth,
                            timeout=request.timeout // 1000
                        )
                    else:
                        
                        resp = requests.post(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            verify=request.SSLVerify,
                            timeout=request.timeout // 1000
                        )
                    end_time = datetime.now()
                elif str.upper(request.method) == "GET":
                    start_time = datetime.now()
                    if request.auth.upper() == "PASSWORD": 
                        resp = requests.get(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            verify=request.SSLVerify,
                            auth=auth,
                            timeout=request.timeout // 1000,
                        )
                    else:
                        resp = requests.get(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            verify=request.SSLVerify,
                            timeout=request.timeout // 1000,
                        )
                    end_time = datetime.now()
                elif str.upper(request.method) == "PUT":
                    start_time = datetime.now()
                    if request.auth.upper() == "PASSWORD": 
                        resp = requests.put(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            auth=auth,
                            verify=request.SSLVerify,
                            timeout=request.timeout // 1000
                        )
                    else:
                        resp = requests.put(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            verify=request.SSLVerify,
                            timeout=request.timeout // 1000
                        )
                    end_time = datetime.now()
                elif str.upper(request.method) == "PATCH":
                    start_time = datetime.now()
                    if request.auth.upper() == "PASSWORD": 
                        resp = requests.patch(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            verify=request.SSLVerify,
                            auth=auth,
                            timeout=request.timeout // 1000
                        )
                    else:
                        resp = requests.patch(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            verify=request.SSLVerify,
                            timeout=request.timeout // 1000
                        )
                    end_time = datetime.now()
                elif str.upper(request.method) == "DELETE":
                    start_time = datetime.now()
                    if request.auth.upper() == "PASSWORD":
                        resp = requests.delete(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            verify=request.SSLVerify,
                            auth=auth,
                            timeout=request.timeout // 1000
                        )
                    else:
                        resp = requests.delete(
                            request.api,
                            headers=request.headers,
                            files=request.file,
                            data=request.body,
                            verify=request.SSLVerify,
                            timeout=request.timeout // 1000
                        )
                    end_time = datetime.now()
                else:
                    raise Exception("invalid method")

                obj.status_code = resp.status_code
                obj.response_body = resp.text
                obj.response_headers = resp.headers
                obj.response_time = resp.elapsed.total_seconds()

                

                if obj.response_time == 0:
                    elapsed_time = end_time - start_time
                    obj.response_time = elapsed_time.total_seconds()
                logging.info(f"URL: {request.api}")
                logging.info(f"Time elapsed: {obj.response_time} secs")
                
                result = obj
                
        except Exception as e:
                print(traceback.format_exc())
                print(str(e))
                result = obj
            
            # add retries if required
        if request.retries > 0:
                if result.status_code not in request.expected_code_list:
                    request.retries -= 1
                    logging.info("retrying...........")
                    time.sleep(1)
                    return self.execute_api(request)
        
        return result

                



    def encrypt_(self, password, key):
        try:
            fernet_ = Fernet(key)
            # supress user warnings
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                enc_pass = fernet_.encrypt(password)
        except Exception as e:
            enc_pass = None
            traceback.print_exc()
            print(e)
        return enc_pass

    def decrypt_(self, enc_pass, key):
        try:
            fernet_ = Fernet(key)
            # supress user warnings
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                enc_pass = enc_pass.encode('utf-8')
                dec_pass = fernet_.decrypt(enc_pass)
                dec_pass = dec_pass.decode('utf-8')
        except Exception as e:
            dec_pass = None
            traceback.print_exc()
            print(e)
        return dec_pass


class Request:
    def __init__(self):
        self.api = ""
        self.method = "method"
        self.headers = {}
        self.body = {}
        self.file = ""
        self.SSLVerify = False
        self.credentials = {}
        self.timeout = 30000
        self.auth = ""
        self.expected_code_list = []
        self.retries = 0


class Response:
    def __init__(self, body="", code=0, response_time=0, headers={}):
        self.response_body = body
        self.status_code = code
        self.response_time = response_time
        self.response_headers = headers
